// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <ClanLib/clanlayer2.h>

#include "PingusSound.hh"
#include "globals.hh"
#include "plf_datatypes.hh"
#include "Pingu.hh"
#include "ButtonPanel.hh"
#include "actions/pingu_actions.hh"

CL_Surface* ButtonPanel::button_cap;

ButtonPanel::ButtonPanel(PLF* plf)
{
  need_redraw = true;
  last_press = 0;
  pressed_button = 2;

  vector<button_data> buttons = plf->get_buttons();

  for(vector<button_data>::size_type i=0; i < buttons.size(); ++i) {
    button.push_back(new ActionButton(38 * i,
				      CL_Display::get_height() - 56, buttons[i].name));
  }

  armageddon_sur = CL_Res_Surface::load("Buttons/armageddon", *resource());
  pause_sur = CL_Res_Surface::load("Buttons/pause", *resource());
  forward_sur = CL_Res_Surface::load("Buttons/forward", *resource());

  button_cap      = CL_Res_Surface::load("Buttons/button_cap", *resource());

  forward_pressed = false;
  pause_pressed = false;
  armageddon_pressed = false;
  left_pressed = 0;
}

ButtonPanel::~ButtonPanel()
{
}

void
ButtonPanel::let_move()
{
  button[pressed_button]->let_move();

  {
    int temp_button = pressed_button;
    
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F1)) {
      pressed_button = 0;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F2)) {
      pressed_button = 1;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F3)) {
      pressed_button = 2;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F4)) {
      pressed_button = 3;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F5)) {
      pressed_button = 4;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F6)) {
      pressed_button = 5;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F7)) {
      pressed_button = 6;
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_F8)) {
      pressed_button = 7;
    }
  
    if (pressed_button >= button.size())
      pressed_button = temp_button;
  }

  if (CL_Mouse::left_pressed()) {
    int mouse_x = CL_Mouse::get_x();
    int mouse_y = CL_Mouse::get_y();
    int temp_button;

    last_press = CL_System::get_time();
    
    // Get the pressed mouse button
    if (mouse_y > CL_Display::get_height() - 56
	&& mouse_x < ((button.size() + 3) * 38))
      {
	temp_button = mouse_x / 38;
	if (temp_button == button.size()) {
	  if (armageddon_pressed == 0) {
	    armageddon_pressed = 1;
	  } else if (armageddon_pressed == 2) {
	    armageddon_pressed = 3;
	  } 
	  if (armageddon_pressed == 3) {
	    world->armageddon();
	  }
	} else if (temp_button == button.size() + 1) {
	  if (!left_pressed) {
	    pause_pressed = !pause_pressed;
	    server->toggle_pause();
	  }
	} else if (temp_button == button.size() + 2) {
	  if (!left_pressed) {
	    forward_pressed = !forward_pressed;
	    server->toggle_fast_forward();
	    client->toggle_fast_forward();
	  }
	} else {
	  pressed_button = temp_button;
	}
      }
    left_pressed = true;
  } else {
    if (armageddon_pressed <= 2) {
      if (last_press + 350 < CL_System::get_time()) {
	armageddon_pressed = 0;
      } else if (armageddon_pressed == 1) {
	armageddon_pressed = 2;
      }
    }
    left_pressed = false;
  }
}

string
ButtonPanel::get_action_name()
{
  return button[pressed_button]->get_action_name();
}

void 
ButtonPanel::draw() 
{
  // FIXME: This could be optimized
  need_redraw = true;
  if (need_redraw) {
    // draw the buttons
    for(vector<ActionButton*>::size_type i=0; i < button.size(); ++i) {
      button[i]->draw();
    }
    armageddon_sur->put_screen(38 * button.size(), CL_Display::get_height() - 56);
    pause_sur->put_screen(38 * (button.size() + 1), CL_Display::get_height() - 56);
    forward_sur->put_screen(38 * (button.size() + 2), CL_Display::get_height() - 56);

    if (armageddon_pressed)
      button_cap->put_screen((button.size()) * 38, CL_Display::get_height() - 56);
    
    if (pause_pressed) 
      button_cap->put_screen((button.size() + 1) * 38, CL_Display::get_height() - 56);
    
    if (forward_pressed) 
      button_cap->put_screen((button.size() + 2) * 38, CL_Display::get_height() - 56);

    need_redraw = false;
  }

  // draw the capture rectangle for the active button
  button_cap->put_screen(pressed_button*38, CL_Display::get_height() - 56);
}

void
ButtonPanel::set_server(Server* s)
{
  server = s;
  world = server->get_world();
}

void
ButtonPanel::set_client(Client* c)
{
  client = c;
}

/* EOF */
