// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <string>

#include "PingusLevelDesc.hh"

PingusLevelDesc::PingusLevelDesc(PLF* plf)
{
  typedef string::size_type ST;
  ST pos, pos_next;
  string desc_string;

  desc_string = plf->get_description();
  levelname = plf->get_levelname();

  // BUG: Begin, this is very buggy and don't work in all situations
  pos = 0;
  pos_next = desc_string.find("\n", 0);
  
  if (pos_next == string::npos) {
    description.push_back(desc_string);
  }

  while(pos_next != string::npos)
    {
      description.push_back(desc_string.substr(pos, pos_next - pos));
      
      pos = pos_next + 1;
      if (pos >= desc_string.size())
	break;

      pos_next = desc_string.find("\n", pos + 1);
    }
  if (pos_next == string::npos) {
    description.push_back(desc_string.substr(pos, desc_string.size() - pos));
  }
  // BUG: End
  
  background = CL_Res_Surface::load("Textures/stone", *resource());
  font       = CL_Res_Font::load("Fonts/courier_h", *resource());
  title      = CL_Res_Font::load("Fonts/clanfont", *resource());
}

void
PingusLevelDesc::draw(PingusLevelDesc::LoadingStatus status)
{
  CL_Display::clear_display();
  background->put_screen(0, 0, CL_Display::get_width(), CL_Display::get_height());
  CL_Display::fill_rect(0, 0, CL_Display::get_width(), CL_Display::get_height(), 0.0, 0.0, 0.0, 0.5);
  title->print_center(CL_Display::get_width() / 2, 50, levelname.c_str());

  for(unsigned int i=0; i < description.size(); ++i)
    font->print_center(CL_Display::get_width() / 2 - 20, 100 + (20 * i), description[i].c_str());


  if (status == LOADING) {
    font->print_center(CL_Display::get_width() / 2 - 20, CL_Display::get_height() - 80, "Loading...");
  } else {
    font->print_center(CL_Display::get_width() / 2 - 20, CL_Display::get_height() - 80, "Loading finished, press any key to start.");
  }

  CL_Display::flip_display();
}

/* EOF */
