// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "PingusLevelDesc.hh"
#include "GameTime.hh"
#include "globals.hh"

#include "TrueServer.hh"

TrueServer::TrueServer(PLF* level_data, Result& r)
{
  local_game_speed = game_speed;
  world = 0;
  restart(level_data, r);
}

TrueServer::TrueServer()
{
  world = 0;
}

TrueServer::~TrueServer()
{
}

void
TrueServer::let_move(void)
{
  if (enough_time_passed() || CL_Keyboard::get_keycode(CL_Keyboard::KEY_SPACE)) {
    GameTime::increase();
    Server::let_move();
    world->let_move();
  }
}

void
TrueServer::restart(PLF* level_data, Result& r)
{
  PingusLevelDesc leveldesc(level_data);

  fast_forward = false;
  pause = false;
  last_time = 0;
  local_game_speed = game_speed;

  if (world)
    delete world;

  world = new World;
  leveldesc.draw(PingusLevelDesc::LOADING);
  world->init(level_data, r);
  leveldesc.draw(PingusLevelDesc::FINISHED);
  GameTime::reset();
}

bool
TrueServer::enough_time_passed(void)
{
  if (pause) {
    return false;
  }

  if (fast_forward) {
    // BUG: This should skip some frames, so it also works on slower
    // machines
    last_time = CL_System::get_time();
    return true;
  } else {
    if (last_time + local_game_speed > CL_System::get_time())
      return false;
    else {
      last_time = CL_System::get_time();
      return true;
    }
  }
}

void
TrueServer::toggle_fast_forward(void)
{
  fast_forward = !fast_forward;
  if (fast_forward)
    local_game_speed = 5;
  else 
    local_game_speed = game_speed;
}

void 
TrueServer::toggle_pause(void)
{
  pause = !pause;
}

/* EOF */
