// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "smasher.hh"
#include "../actions/smashed.hh"

Smasher::Smasher(trap_data data)
{
  smashing = false;
  x_pos = data.x_pos;
  y_pos = data.y_pos;
  z_pos = data.z_pos;
  surface = CL_Res_Surface::load("Traps/smasher", *resource());
  counter.set_size(surface->get_no_sprs());
  counter.set_type(GameCounter::once);
  counter.set_speed(1);
  counter = surface->get_no_sprs() - 1;
}

Smasher::~Smasher()
{
}

void
Smasher::let_move()
{
  if (smashing)
    ++counter;
}

void 
Smasher::draw_offset(int x, int y)
{
  assert(surface);
  surface->put_screen(x_pos + x, y_pos + y, counter);
}

bool 
Smasher::catch_pingu(Pingu* pingu)
{
  if (!pingu->is_alive())
    return false;

  if (counter == surface->get_no_sprs() -1) 
    smashing = false;
  
  if (pingu->get_x() > x_pos + 20 && pingu->get_x() < x_pos + 177 + 20
      && pingu->get_y() > y_pos + 100 && pingu->get_y() < y_pos + 150) 
    {
      if (!smashing) {
	counter = 0;
	smashing = true; 
      }

      if (counter >= 5 && counter <= 7) {
	pingu->set_action(new Smashed(), -1);
      }
      return true;
    } else {
      return false;
    }
}

/* EOF */
